 
 package Def124;

################################################################
###                                                          ###
### SCaFoS                                                   ###
### Gestion des sequences par defaut                         ###
###                                                          ###
################################################################
### Version 4.40 du 2006/08/31                           BPH ###
################################################################


 use strict;
 use warnings;
 use Cwd;

 use Var124;
 use Lib124;
 use Gub124;
 
 use vars qw(@ISA @EXPORT);
 require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = qw/ ReadDef CloseDef LetUserDef SearchOneSeqDef /;

 
################################################################


 my $bAff = '';       # texte pour affichage en mode graphique
 my $L;
 my $KeyFile;


################################################################
### Lecture du fichier .def 
sub ReadDef {
    ($KeyFile) = @_;
    my ($SeqDef,$lSeqDefCur);
    my ($Ngrp, $LDef, $L2);
    
    # utilisation et maj des fichiers .def : ouverture du fichier en lecture
    $lSeqDef = "<<<<<";
    $lSeqDefCur = "<<<<<";
    if ( open(DEF, $FileRep.$SepDir.$KeyFile.".def") ) {
        # lecture du fichier
        while ( $L = <DEF> ) {
            if ( $L =~ m/^>(.*)(\s{5})#(.+)/ ) {
                $SeqDef = $1;
                $L2 = $3;
                $SeqDef =~ s/^\s+//;
                $SeqDef =~ s/\s+$//;
                $lSeqDef .= ($SeqDef . "<<<<<");
                if ( $L2 =~ m/(.+):(.+)/ ) {
                    $Ngrp = $1;
                    $LDef = $2;
                    $Ngrp =~ s/^\s+//;
                    $Ngrp =~ s/\s+$//;
                    $LDef =~ s/^\s+//;
                    $LDef =~ s/\s+$//;
                    $hSeqDef{$Ngrp} = $LDef;
                    }
                }
            }
        close DEF;
        }
    # creation de nouveaux fichiers .def
    if ( !open(DEF, "> $CefRep$KeyFile.def") ) { AffError("Can't open for writing file \'$CefRep$KeyFile.def\' !"); }
    # ecriture de l'entete
    print DEF "# In directory : $FileRep\n";
    print DEF "# Out directory : $CefRep\n";
    print DEF "# ".scalar(localtime(time()))." \n\n";
}


sub CloseDef {
    close DEF;
}

### GESTION DES SEQUENCES PAR DEFAUT ###########################
# NOM : SearchOneSeqDef
# DESCRIPTION : verifie si une et une seule sequence existe dans
#     le .DEF pour le groupe en cours de traitement
# PARAMETRES : Nom du fichier .ALI
#              Nom du groupe
# RETOUR : Drapeau   0 -> repousse la demande a l'utilisateur
#                    1 -> 1 sequence trouvee dans DEF : on la prend
#                   -1 -> Choix par SCaFoS
#          Sequence
#          Nom sequence
sub SearchOneSeqDef {
    my ($KeyFile, $Grp) = @_;
    my ($Seq, $NameSeq, $bList, $bList2, $lSeqDefCur) = ('', '', '', '');
    my ($nb, $flag) = (0, 0);
    my $buf;
    my (@T1, @T2);
    my ($Color, $mean, $nbIn);

    #** determine nb de sequences de DEF correspondant au groupe ***
    @T1 = split('\|', $hSeqFile{$KeyFile}{$Grp}[3]);
    foreach $buf ( @T1 ) {
        if ( $lSeqDef =~ m/<<<<<$buf<<<<</ ) {
            $NameSeq = $buf;
            $nb += 1;
            $bList .= "\n       ($nb) ".$NameSeq;
            }
        }

    #** aucune sequence dans DEF : on verifie dans CEF le nombre de sequences possibles ***
    if ( $nb == 0 ) {
NOSEQ:
        if ( $FlagDefAuto == 0 )
            { $flag = 0;  }
        else
            { $flag = -1; } 
        }
    
    else {
        if ( !exists($hSeqDef{$Grp}) ) { goto NOSEQ; }
        #** de nouvelles sequences pour ce groupe ***
        @T2 = split(',', $hSeqDef{$Grp});
        if ( $#T1 > $#T2 ) {
            if ( $FlagDefAuto == 0 ) {
                # demande confirmation a l'utilisateur
                $flag = 0;
                goto NEWSEQ;
                }
            else {
                # prend la sequence determinee par CEF
                $flag = -1;
                }
            }

        #** toutes les sequences du .ali sont-elles dans .def ? ***
        else {
            $bList2 = '|';
            for $buf ( @T2 ) {
                $buf =~ s/^\s+//;
                $buf =~ s/\s+$//;
                $bList2 .= "$buf\|";
                }
            $buf = $hSeqDef{$Grp};
            $buf =~ s/^\s+//;
            $buf =~ s/\s+$//;
            $buf =~ s/\s+,/,/g;
            $buf =~ s/,\s+/,/g;
            $bList2 =~ s/,/\|/g;
            # si les sequences ne sont pas les memes : utilisateur choisit
            $buf = '|'.$hSeqFile{$KeyFile}{$Grp}[3];
            if ( $bList2 ne $buf ) {
                $nb = 0;
                for ( @T1 ) 
                    { if ( $bList2 =~ m/$_/ ) { $nb += 1;} }
                if ( $nb == ($#T2+1) ) { goto ONESEQ; }
                }       

            # une et une seule sequence dans DEF pour ce groupe : on la prend
            elsif ( $nb == 1 ) {
ONESEQ:            
                # determiner sequence correspondante
                $Seq = SearchSeqFromName($NameSeq, $KeyFile, $Grp);
                # ajoute la sequence dans DEF
                AddFileDef($NameSeq, $Grp, @{$hSeqFile{$KeyFile}{$Grp}}[3]);    
                $flag = 1;
                }
            # plusieurs sequences de DEF pour ce groupe
            else {
                # demande confirmation a l'utilisateur
                if ( $FlagDefAuto == 0 )
                    {
                    if ( @{$hSeqFile{$KeyFile}{$Grp}}[4] == 0 )
                        # Choix par SCaFoS si toutes les sequences sont identiques
                        { $flag = -1; }
                    else {
NEWSEQ:
                        # calcul de la distance moyenne hors groupe -> couleur d'affichage
                        $mean = 0;
                        @T1 = split("\n", @{$hSeqFile{$KeyFile}{$Grp}}[1]);
                        for $buf ( @T1 ) {
                            if ( !($buf =~ m/^(\d+)\s+(.*)/) ) { next; }
                            $NameSeq = SearchGrpSpe($1, $KeyFile, 1);
                            if ( $NameSeq eq "" ) { next; }
                            $mean += @{$hCorres{$KeyFile}{$NameSeq}}[2];
                            $nbIn++;
                            }
                        $mean /= $nbIn;
                        # affichage fenetre de choix
                        $buf = " List of sequences has changed for the OTU $Grp in $KeyFile\n\n";
                        if ( @{$hSeqFile{$KeyFile}{$Grp}}[4] > ($PctAve * $mean) ) {
                            $buf .= " Maximal ingroup distance is higher than " . 100*$PctAve. "\% of outgroup distance average\n";
                            $Color = 'red';
                            }
                        else {
                            $buf .= " Many sequences in the OTU \'$Grp\' : you could specify a default sequence\n";
                            $Color = "#003F87";
                            }
                        ($flag, $Seq, $NameSeq) = LetUserDef($buf, $KeyFile, $Grp, $Color); 
                        }
                    }
                # prend la sequence determinee par CEF
                else
                    { $flag = -1; }
                }
            }
        }
        
    return($flag, $Seq, $NameSeq);
}


# NOM : LetUserDef
# DESCRIPTION : propose a l'utilisateur de choisir une sequence 
#               qui deviendra la sequence par defaut du groupe courant
# PARAMETRES : Entete du message affiche
#              Nom du fichier .ALI
#              Nom du groupe
# RETOUR : 1 -> choix user             0 -> pas de choix
#          Sequence choisie
#          Nom de la sequence choisie
sub LetUserDef {
    my ($bMsg, $KeyFile, $Grp, $Color) = @_;
    my @ListSeq;
    my $nb = 0;
    my %h = ();
    my %hTmp;
    my $buf;
    my $n;
    my $bList = '';
    
    my $s;
    my $Id;   
    my %hBlc = ();
    my @LS;
    
    $bMsg = "$KeyFile :\n$bMsg";
    @LS = split("\n", @{$hSeqFile{$KeyFile}{$Grp}}[1]);         # Id/Seq\n...
    
    # liste des sequences du groupe avec distance hors groupe
    @ListSeq = split('\|', @{$hSeqFile{$KeyFile}{$Grp}}[3]);    # NomSeq1|NomSeq2...
    foreach ( @ListSeq ) {
        $buf = @{$hCorres{$KeyFile}{$_}}[2];
        $h{$_} = $buf;
        $Id = @{$hCorres{$KeyFile}{$_}}[0];
        # nb caracteres manquants par sequences
        foreach $s ( @LS ) {
            $s =~ m/^(\d+)\s+(.*)/;
            if ( $1 == $Id ) {
                $buf = $2;                         # seq commence en 11
                $buf =~ s/ |X|-|\*|\.|\$/\?/g;     # remplace ., $, *, espace ou X par ?
                $n=0;
                $n++ while ( $buf =~ m/\?/g );     # compte nombre total ?
                $hBlc{$_} = $n;
                last;
                }
            }
        }
    
    
    # tri des sequences par distante croissante
    $nb = 0;
    $bList = pack("A56A10A10", '', "distance", "miss.pos");
    if ( $FlagCmp == 1 ) { $bList .= "compos."; }
    $bList .= "\n";
    foreach ( sort { $h{$a} <=> $h{$b} || $a cmp $b } keys %h ) {
        $hTmp{++$nb} = $_;
        $bList .= pack("A5A48A5A5A4", "($nb)", $_, $h{$_}, '', $hBlc{$_});
        if ( $FlagCmp == 1 )
            # ajoute composition
            { $bList .= pack("A6A5", '', @{$hCorres{$KeyFile}{$_}}[1])."\n"; }
        else
            { $bList .= "\n"; }
        }
    if ( $FlagIHM == 0 ) { print $bMsg . $bList; }
    $n = -1;
    while ( ($n < 0) or ($n > $nb) ) {
        if ( $FlagIHM == 1 ) {
            $buf = -2;
            while ( $buf == -2 ) {
                $buf = ViewDefaultSeq($bMsg, $bList, 1, $Color);
                if ( $buf == -2 ) 
                    { ViewInfo("Warning", "With this option, you must choose a sequence !", 'red'); }
                }
            }
        else {
            if ( $Color eq 'red' ) 
                { print "  (0) confirm elliminating of this OTU\n\n Let go SCaFoS or choose one sequence > "; }
            else
                { print "  (0) Let compute SCaFoS for this OTU\n\n Let go SCaFoS or choose one sequence > "; }
            $buf = <STDIN>;
            print "\n";
            }
        $buf =~ s/\s+//g;
        if ( $buf eq "" ) { $buf = "0"; }
        $n = int($buf);
        # choix utilisateur
        if ( ($n > 0) and ($n <= $nb) ) { 
            # ajoute la sequence dans DEF
            AddFileDef($hTmp{$n}, $Grp, @{$hSeqFile{$KeyFile}{$Grp}}[3]);
            # retourne la sequence
            return(1, SearchSeqFromName($hTmp{$n}, $KeyFile, $Grp), $hTmp{$n});
            }
        # choix de CEF / pas de sequence               
        elsif ( $n == 0 or $n == -1)
            { return($n, "", ""); }
        # mauvaise saisie
        else
            { $n = -1; }
        }
    return(0, "", "");
}


# NOM : AddFileDef
# DESCRIPTION : Ajoute sequence selectionnee dans le fichier .def
# PARAMETRES : Nom de la sequence
#              Nom du fichier .ALI
#              Nom du groupe
sub AddFileDef {
    my ($NameSeq, $Grp, $buf) = @_;
    $buf =~ s/\|/, /g;
    print DEF ">$NameSeq     # $Grp : $buf\n";
    if ( !($lSeqDef =~ m/<<<<<$NameSeq<<<<</) )
        { $lSeqDef .= ($NameSeq . "<<<<<"); }
}


# NOM : SearchSeqFromName
# DESCRIPTION : recherche une sequence dans hSeqFile a partir du
#     nom de la sequence 
# PARAMETRES : Nom de la sequence
#              Nom du fichier .ALI
#              Nom du groupe
# RETOUR : Sequence trouvee ou chaine vide
sub SearchSeqFromName {
    my ($NameSeq, $KeyFile, $Grp) = @_;
    my $S;
    my @ListSeq;
    my $buf;
    my $L;
    
    @ListSeq = split("\n", @{$hSeqFile{$KeyFile}{$Grp}}[1]);
    for $S ( @ListSeq ) {
        if ( $S =~ m/^(\d+)\s+(.*)/ ) {
            ($buf, $L, $L) = SearchGrpSpe($1, $KeyFile, 1);
            if ( $buf eq $NameSeq )
                { return($2); }
            }
        }
    return('');
}


1;

### fin du fichier                                           ###
################################################################
