
 package Lib124;

################################################################
###                                                          ###
### SCaFoS                                                   ###
### fonctions communes de SCaFoS                             ###
###                                                          ###
################################################################
### Version 4.43 du 2006/11/26                           BPH ###
################################################################


 use strict;
 use warnings;

 use Var124;
 use Gub124;
 
 use vars qw(@ISA @EXPORT);
 require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = qw/ SeqType UnconvenientOut Get_NameCef TooShort CountPosMis AffError EndLine CreateDir RemoveDir SearchSpecName SearchGrpSpe /;



################################################################
# NOM : SeqType 
# DESCRIPTION : Determine si la sequence est de type proteine ou acide nucleique
# PARAMETRES : la sequence a tester
# RETOUR : protein ou DNA
sub SeqType {
    my ($Seq) = @_;
    my $nbACGTU = 0;
    my $nbOtherChar = 0;
    
    if ( $Seq =~ m/[EFILPQ]/i ) { return('protein'); }
    $nbACGTU++ while ( $Seq =~ m/[ACGTU]/ig );
    $nbOtherChar++ while ( $Seq =~ m/[RYSWKMBDHV]/ig );
    if ( $nbACGTU / ($nbACGTU+$nbOtherChar) < 0.90 ) { return('protein'); }
    else { return('DNA'); }
}

  
################################################################
# NOM : UnconvenientOut 
# DESCRIPTION : Verifie que le repertoire de sortie n'est pas la racine ou un home 
# PARAMETRES : le nom du repertoire de sortie
# RETOUR : 0 si OK -1 sinon 
sub UnconvenientOut {
    my ($Out) = @_;
    
    # $Out != ~ / /home/XXXX
    $Out =~ s/^\s+//g;
    $Out =~ s/\s+$//g;
    if ( $Out eq '~' or $Out eq '/' or $Out eq '~/' or $Out =~ m/\/home\/\S*\/$/ or $Out =~ m/\/home\//) { 
        if ( $FlagIHM == 1 ) {
            ViewInfo('Stop', "You can't use $Out as output directory", 'red');
            return(-1);
            }
        else {
            die "\n You can't use $Out as output directory\n\n ";
            }
        }
    else {
        return(0);
        } 
}

 
################################################################
# NOM : Get_NameCef 
# DESCRIPTION : Determine le nom du fichier de concatenation
# PARAMETRES : so
# RETOUR : Nom du fichier
sub Get_NameCef {    
    ($CefRep) = @_;
    my ($L, $L2);
    
    if ( $CefRep eq "." )
        { return($FileRep."scafos"); }
    else {
        $L = rindex($CefRep, $SepDir);
        if ( $L == length($CefRep)-1 ) {
            $L2 = rindex($CefRep, $SepDir, $L-1);
            if ( $L2 >= 0 ) 
                { return($CefRep.substr($CefRep, $L2+1, $L-$L2-1)); }
            elsif ( $L2 == -1 )
                { return($CefRep.substr($CefRep, 0, $L)); }
            }
        elsif ( $L > -1 ) 
            { return($CefRep.substr($CefRep, $L+1, length($CefRep)-$L)); }
        else
            { return($CefRep.$CefRep); }
        }   
}        


################################################################
# NOM : TooShort
# DESCRIPTION : Verification longueur reelle sequence
# PARAMETRES : Nb max de 'non-caracteres' autorises, sequence
# RETOUR : 1 si le nombre de 'non-caracteres' dans $Seq est superieur ou egal a $MaxMiss, sinon 0
#          Nb de 'non-caracteres'
sub TooShort {
    my ($MaxMiss, $Seq) = @_;
    my $nb = 0;
    
    $nb++ while ( $Seq =~ m/\?|\*|\s|\$|\.|-|X/g );
    if ( $nb <= $MaxMiss ) { return(0, $nb); }
    else { return(1, $nb); }
}


################################################################
# NOM : CountPosMis 
# DESCRIPTION : Decompte des positions manquantes (?| |X|*|.|-|$)
# PARAMETRES : sequence
# RETOUR : Nombre de 'non-caracteres'
sub CountPosMis {
    my $n = 0;
    my ($b) = @_;
    
    $n++ while ( $b =~ m/\?|\s|\*|\.|\$|-|X/g );
    return($n);
}


################################################################
# NOM : AffError
# DESCRIPTION : Affichage un message d'erreur et ferme l'application
# PARAMETRES : Message a afficher
# RETOUR : Ferme application
sub AffError {
    my ($bAff) = @_;
    
    if ( $FlagIHM == 1 ) {
        ViewInfo('Warning' , $bAff, 'red');
        print "\n\n SCaFoS has encountered a problem with your dataset :\n   $bAff\n\n";
        exit(-1);
        }
    else
        { die "\n$bAff\n\n"; }
}


### Gestion des fins de lignes dos/unix ########################
sub EndLine {
    my ($Line) = @_;
    my $b = "";
    my ($n1, $n2);

    $n1 = index($Line, "\r");
    # \r en debut de chaine
    if ( $n1 == 0 ) {
        $n1 = (substr($Line,1,1) eq "\n" ) ? 2 : 1;
        $n2 = index(substr($Line, $n1), "\r");
        $b = substr($Line, $n1, $n2 - $n1 - 1);
    }
    # \r dans la chaine
    elsif ( $n1 != -1 ) {
        $n1 = (substr($Line,0,1) eq "\n" ) ? 1 : 0;
        $n2 = index(substr($Line, $n1), "\r");
        if ( $n2 != -1 )
            { $b = substr($Line, $n1, $n2 - $n1); }
        else
            { $b = substr($Line, $n1, length($Line) - $n1); }
    }
    # pas de \r
    else {
        $n1 = (substr($Line,0,1) eq "\n" ) ? 1 : 0;
        $n2 = index(substr($Line, $n1), "\n");
        if ( $n2 != -1 )
            { $b = substr($Line, $n1, $n2 - $n1); }
        else
            { $b = substr($Line, $n1, length($Line) - $n1); }
    }
    return $b;
}


################################################################
# NOM : CreateDir
# DESCRIPTION : creation d'un repertoire 
# PARAMETRES : nom du repertoire
# RETOUR : quitte l'application
sub CreateDir {
    my ($Dir) = @_;
#print "$Dir\n";
    if ( !mkdir($Dir) )
#    system "mkdir $Dir";
        { AffError("Can\'t create directory \'$Dir\'"); }
#system("dir $Dir");
}


# NOM : RemoveDir
# DESCRIPTION : suppresion d'un repertoire et de ses sous-repertoires
# PARAMETRES : nom du repertoire
# RETOUR : na
sub RemoveDir {
    my ($Dir) = @_;
    my @T1;
    my $D;

    # suppression des fichiers
    if ( $^O eq 'MSWin32' )
        { system("del /s /q ".$Dir.'*'); }
    else
        { system("rm -f ".$Dir.'*>/dev/null'); }
    # appel recursif des sous-repertoires
    opendir REP, $Dir;
    @T1 = readdir REP;
    closedir REP;
    foreach $D (sort @T1) {
        if ( $D ne '.' and $D ne '..') { RemoveDir($Dir.$D.$SepDir); }
        }
    # suppression du repertoire vide
    rmdir($Dir);
}


################################################################
# NOM : SearchSpecName
# DESCRIPTION : Decompose Nom d'espece et Numero d'acces
# PARAMETRES : chaine contenant les donnees, separateur
# RETOUR : Nom d'espece
sub SearchSpecName {
    my ($buf, $car) = @_;
    my $SpecName = "";        # nom de la sequence courante sans extension

    my $n = index($buf, $car);
    if ( $n == -1 )
        { $SpecName = $buf; }
    else
        { $SpecName = substr($buf, 0, $n); }
    return $SpecName;
}


### recherche la liste des identifiants des especes appartenant
### au meme groupe que l'espece courante #######################
sub SearchGrpSpe {
    my ($Id, $KeyFile, $flag) = @_;  # $Id= identifiant de la sequence courante
                                     # $KeyFile= nom du fichier .ali
                                     # $flag= 1:cherche unique nom sequence, sinon 0
    my $SpecName = "";        # nom de l'espece courante
    my $SeqName = "";         # nom unique d'identification de la sequence
    my $GrpName = "";         # nom du groupe auquel appartient la sequence

    # recherche le nom de l'espece courante
    my $S;
    for $S ( keys %{$hCorres{$KeyFile}} ) {
        if ( length($S) == 0 ) { next; }
        if ( ${$hCorres{$KeyFile}{$S}}[0] == $Id ) {
            $SpecName = SearchSpecName($S, "@");
            $SeqName = $S;
            last;
            }
        }
    if ( $flag == 1 ) 
        { return($SeqName); }
        
    # recherche le groupe auquel appartient l'espece courante
    my ($b1, $G) = ("", "");
    my @Grp = sort keys %{$hSeqFile{$KeyFile}};  
    for $G ( @Grp ) {
        if ( length($G) == 0 ) { next; }
        $b1 = "|" . @{$hSeqFile{$KeyFile}{$G}}[3];
        if ( $b1 =~ m/\|$SeqName\|/ ) {
            $GrpName = $G;
            last;
        }
    }
    
    # construit la liste de l'ordre des especes du groupe dans .puz
    my $PosList = "\|";
    for ( @{$hSeqFile{$KeyFile}{$GrpName}[5]} ) {
        $PosList .= ($hIdent{$KeyFile}{$_}."\|");
    }
    
    return ($SeqName, $GrpName, $PosList);
}


1;


### fin du fichier                                           ###
################################################################
